# LDAP audit tools

This python package provide a collection of tools usefull to audit a LDAP directory.

Several types of analysis are possible on the same way from a LDAP directory or a LDIF file.

## ldap-search

Tool to search entries from LDAP directory or LDIF file:
- You could filter entries using `-b/--base`, `-s/--scope` and `-f/--filter` parameters
- Optionally, you could export matching entries and their attributes in a CSV file.

```
usage: ldap-search [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING] [-H LDAP_URI]
                   [-D LDAP_BINDDN] [-w LDAP_BINDPWD] [-F LDAP_FILTER] [-2] [-P]
                   [--ldap-page-size LDAP_PAGE_SIZE] [-i INPUT] [-b BASE] [-B] [-s SCOPE]
                   [-f FILTERS] [-O] [-o OUTPUT] [--csv-delimiter CSV_DELIMITER]
                   [--csv-quotechar CSV_QUOTECHAR] [--csv-encoding CSV_ENCODING] [-M MAXLEN]
                   [attrs ...]

Tool to search entries from LDAP server of from LDIF file

positional arguments:
  attrs                 Attribute(s) name

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)
  -M MAXLEN, --max-length MAXLEN
                        Attributes's values max length

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.

CSV output options:
  -o OUTPUT, --output OUTPUT
                        Output CSV file path
  --csv-delimiter CSV_DELIMITER
                        CSV delimiter (default: ';')
  --csv-quotechar CSV_QUOTECHAR
                        CSV quotechar (default: '"')
  --csv-encoding CSV_ENCODING
                        CSV encoding (default: 'utf-8')
```

## list-objectclass-and-attribute

Tool to analyse objectClasses and attributes used in a LDAP directory or a LDIF file. You could
filter entries to audit using `-b/--base`, `-s/--scope` and `-f/--filter` parameters.

```
usage: list-objectclass-and-attribute [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING]
                                      [-H LDAP_URI] [-D LDAP_BINDDN] [-w LDAP_BINDPWD]
                                      [-F LDAP_FILTER] [-2] [-P]
                                      [--ldap-page-size LDAP_PAGE_SIZE] [-i INPUT] [-b BASE]
                                      [-B] [-s SCOPE] [-f FILTERS] [-O]
                                      [--check-objectclass [CHECK_OBJECTCLASS ...]]
                                      [--check-attribute [CHECK_ATTRS ...]] [-N NBEXAMPLE]
                                      [-L EXAMPLE_MAX_LEN]

Tool to analyse objectClasses and attributes used in LDIF file

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)
  --check-objectclass [CHECK_OBJECTCLASS ...]
                        List of objectclass that object must have
  --check-attribute [CHECK_ATTRS ...]
                        List of attributes that object must have
  -N NBEXAMPLE, --nb-example NBEXAMPLE
                        Number of example to retreived for each attribute
  -L EXAMPLE_MAX_LEN, --example-max-length EXAMPLE_MAX_LEN
                        Max length of example value retreived for each attribute

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.
```

## list-attribute-nomenclature

Tool to list attribute nomenclature values from a LDAP directory of a LDIF file:
- You could filter entries to audit using `-b/--base`, `-s/--scope` and `-f/--filter` parameters
- Optionally, you could export nomenclature in a CSV file.

```
usage: list-attribute-nomenclature [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING]
                                   [-H LDAP_URI] [-D LDAP_BINDDN] [-w LDAP_BINDPWD]
                                   [-F LDAP_FILTER] [-2] [-P]
                                   [--ldap-page-size LDAP_PAGE_SIZE] [-i INPUT] [-b BASE]
                                   [-B] [-s SCOPE] [-f FILTERS] [-O] [-o OUTPUT]
                                   [--csv-delimiter CSV_DELIMITER]
                                   [--csv-quotechar CSV_QUOTECHAR]
                                   [--csv-encoding CSV_ENCODING] [-N NBEXAMPLE] [-S] [-R]
                                   attr

Tool to list attribute nomenclature values from LDIF file

positional arguments:
  attr                  Attribute name

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)
  -N NBEXAMPLE, --nb-example NBEXAMPLE
                        Number of object DN to retreived for each attribute value
  -S, --sort-by-occurences
                        Sort result by number of occurences of the values
  -R, --reverse         Reverse result ordering

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.

CSV output options:
  -o OUTPUT, --output OUTPUT
                        Output CSV file path
  --csv-delimiter CSV_DELIMITER
                        CSV delimiter (default: ';')
  --csv-quotechar CSV_QUOTECHAR
                        CSV quotechar (default: '"')
  --csv-encoding CSV_ENCODING
                        CSV encoding (default: 'utf-8')
```

## list-duplicated-attribute-values

Tool to list duplicated attribute(s) values from a LDAP directory of a LDIF file :
- You could audit one or multiple attributes values
- You could filter entries to audit using `-b/--base`, `-s/--scope` and `-f/--filter` parameters
- Optionally, you could export duplicated values with DN of corresponding objects in a CSV file

```
usage: list-duplicated-attribute-values [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING]
                                        [-H LDAP_URI] [-D LDAP_BINDDN] [-w LDAP_BINDPWD]
                                        [-F LDAP_FILTER] [-2] [-P]
                                        [--ldap-page-size LDAP_PAGE_SIZE] [-i INPUT]
                                        [-b BASE] [-B] [-s SCOPE] [-f FILTERS] [-O]
                                        [-o OUTPUT] [--csv-delimiter CSV_DELIMITER]
                                        [--csv-quotechar CSV_QUOTECHAR]
                                        [--csv-encoding CSV_ENCODING] [-S] [-R]
                                        attrs [attrs ...]

Tool to list attribute duplicated values from LDIF file

positional arguments:
  attrs                 Attribute name(s). You could specify multiple attribute names.

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)
  -S, --sort-by-occurences
                        Sort result by number of duplicated values
  -R, --reverse         Reverse result ordering

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.

CSV output options:
  -o OUTPUT, --output OUTPUT
                        Output CSV file path
  --csv-delimiter CSV_DELIMITER
                        CSV delimiter (default: ';')
  --csv-quotechar CSV_QUOTECHAR
                        CSV quotechar (default: '"')
  --csv-encoding CSV_ENCODING
                        CSV encoding (default: 'utf-8')

```

## list-password-hash-type

Tool to list HASH type of password used in a LDIF file. You could filter entries to audit
using `-b/--base`, `-s/--scope` and `-f/--filter` parameters.

```
usage: list-password-hash-type [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING]
                               [-H LDAP_URI] [-D LDAP_BINDDN] [-w LDAP_BINDPWD]
                               [-F LDAP_FILTER] [-2] [-P] [--ldap-page-size LDAP_PAGE_SIZE]
                               [-i INPUT] [-b BASE] [-B] [-s SCOPE] [-f FILTERS] [-O]
                               [-N NBEXAMPLE]
                               [attribute]

Tool to list password hash types present in LDIF file

positional arguments:
  attribute             Specified password attribute (default: userPassword)

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)
  -N NBEXAMPLE, --nb-example NBEXAMPLE
                        Number of example to retreived for each attribute

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.
```

## detect-clear-password

Tool to detect clear password in a LDAP directory of a LDIF file:
- You could filter entries to audit using `-b/--base`, `-s/--scope` and `-f/--filter` parameters
- Optionally, you could export the list of object(s) DN with clear password in a CSV file.

```
usage: detect-clear-password [-h] [-v] [-d] [-l LOGFILE] [-C] [-p] [-e ENCODING]
                             [-H LDAP_URI] [-D LDAP_BINDDN] [-w LDAP_BINDPWD]
                             [-F LDAP_FILTER] [-2] [-P] [--ldap-page-size LDAP_PAGE_SIZE]
                             [-i INPUT] [-b BASE] [-B] [-s SCOPE] [-f FILTERS] [-O]
                             [-o OUTPUT] [--csv-delimiter CSV_DELIMITER]
                             [--csv-quotechar CSV_QUOTECHAR] [--csv-encoding CSV_ENCODING]
                             [attr]

Tool to detect clear password in LDAP file

positional arguments:
  attr                  Attribute name (default: userPassword)

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         Enable verbose mode
  -d, --debug           Enable debug mode
  -l LOGFILE, --log-file LOGFILE
                        Log file path (default: None)
  -C, --console         Always log on console (even if log file is configured)
  -p, --progress        Enable progress bar
  -e ENCODING, --encoding ENCODING
                        LDAP source encoding (default: utf-8)

LDAP options:
  -H LDAP_URI, --ldap-uri LDAP_URI
                        LDAP host URI
  -D LDAP_BINDDN, --ldap-binddn LDAP_BINDDN
                        LDAP bind DN
  -w LDAP_BINDPWD, --ldap-bindpwd LDAP_BINDPWD
                        LDAP bind password
  -F LDAP_FILTER, --ldap-filter LDAP_FILTER
                        LDAP filter of the search (default: (objectClass*))
  -2, --ldap-v2         Use the version 2 of the LDAP protocol
  -P, --ldap-paged-search
                        Use paged search
  --ldap-page-size LDAP_PAGE_SIZE
                        Page size (for paged search, default: 500)

LDIF options:
  -i INPUT, --input INPUT
                        Input LDIF file

Filtering options:
  -b BASE, --base BASE  Base DN of search
  -B, --exclude-base    Exclude base DN object
  -s SCOPE, --scope SCOPE
                        Scope of the search (must be a postive integer corresponding to max
                        search depth or one of the following: base,one,sub)
  -f FILTERS, --filters FILTERS
                        Filters on object attribute values. Format: [attribute
                        name][!][operator][pattern]. Supported operators: = / > / < / >= / <=
                        / ~= (regex pattern). You could prefix the operator with an
                        exclamation point to inverse the logical and you could filter on
                        count attribute values using count(attr) as attribute name. Multiple
                        filter could be specified by repeating this parameter.
  -O, --or              Combine filters with an OR logical operator instead of AND one.

CSV output options:
  -o OUTPUT, --output OUTPUT
                        Output CSV file path
  --csv-delimiter CSV_DELIMITER
                        CSV delimiter (default: ';')
  --csv-quotechar CSV_QUOTECHAR
                        CSV quotechar (default: '"')
  --csv-encoding CSV_ENCODING
                        CSV encoding (default: 'utf-8')
```

## Locally build (using Docker)

You also could manually build a release using the following command:

```bash
docker run --rm -it -v "$(pwd):/src" -w /src -e DEBFULLNAME -e DEBEMAIL -e GPG_KEY debian:11 ./build.sh --install-deps
```

Release builded files will be store in the `dist` directory.

### Packages signing

By default, builded packages are not signed but you could provided information to allow their signature :

- be sure to define `DEBFULLNAME` and `DEBEMAIL` environment variables
- run the `docker run` command in a context where the `GPG_KEY` environment variable is set as follow :

```bash
GPG_KEY=$( gpg --armor --export-secret-key $DEBEMAIL|base64|tr -d '\n' )
```

**Important :** This solution only work with GPG key **without password** defined. If you have to use a GPG key with a password, you have to remove it before by running the following command :

```
gpg --edit-key $DEBEMAIL
> passwd
> quit
```

Be sure to restore your password after building the Debian packages.

## Copyright

Copyright (c) 2020-2022 Benjamin Renard <brenard@easter-eggs.com>

## License

This program is free software; you can redistribute it and/or modify it under the terms of the GNU
General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not,
write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.
